@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
+ * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
 #include "config.h"
@@ -38,6 +38,9 @@
 #include <X11/Xlib.h>	/* for the ex-pee-arse-geometry routine */
 #include <X11/Xutil.h>	/* for flags returned above */
 
+#ifdef HAVE_GNOME_APPLET
+#include <applet-widget.h>
+#endif
 #include <gnome.h>
 
 #define NELS(a) (sizeof(a) / sizeof(*a))
@@ -57,6 +60,7 @@ typedef struct
 gtop_struct;
 #endif
 
+static char *applet_name = "gstripchart_applet";
 static char *prog_name = "gstripchart";
 static char *prog_version = "1.6";
 static char *config_file = NULL;
@@ -67,11 +71,13 @@ static float slider_filter = 0.0;
 static int include_menubar = 0;
 static int include_slider = 1;
 static int status_outline = 0;
-static int minor_tick=0, major_tick=0;
+static int minor_tick, major_tick;
 static int geometry_flags;
-static int geometry_w=200, geometry_h=50;
+static int slider_width = 10;
+static int geometry_w = 200, geometry_h = 50;
 static int geometry_x, geometry_y;
-static int root_width, root_height;
+static int root_width = 1, root_height;
+static int graphic_display = 1;
 static int update_count = 0;
 static int update_chart = 0;
 
@@ -156,7 +162,7 @@ hi_lo_fmt(double hv, char *hs, double lv, char *ls)
     }
 }
 
-/* 
+/*
  * Expr -- the info required to evaluate an expression.
  *
  * The last and now arrays are doubles in order to handle values that
@@ -192,16 +198,22 @@ Expr;
 static void
 eval_error(Expr *e, char *msg, ...)
 {
+  int len = 0;
+  char str[1000];
   va_list args;
   e->val = 0.0;
   if (update_chart)
     longjmp(e->err_jmp, 1);
   fflush(stdout);
   va_start(args, msg);
-  fprintf(stderr, "%s: %s: ", prog_name, e->eqn_src? e->eqn_src: "");
-  vfprintf(stderr, msg, args);
-  fprintf(stderr, "\n");
+  len = sprintf(str, "%s: %s: ", prog_name, e->eqn_src? e->eqn_src: "");
+  len += vsprintf(str + len, msg, args);
+  fprintf(stderr, "%s\n", str);
   va_end(args);
+
+  if (graphic_display)
+    gnome_dialog_run(GNOME_DIALOG(gnome_error_dialog(str)));
+
   exit(EXIT_FAILURE);
 }
 
@@ -252,12 +264,7 @@ Gtop_data;
  * referenced, and so the corresponding libgtop routine should be
  * called on each eval cycle.
  */
-static int gtop_cpu;
-static int gtop_mem;
-static int gtop_swap;
-static int gtop_uptime;
-static int gtop_loadavg;
-static int gtop_netload;
+static int gtop_cpu, gtop_mem, gtop_swap, gtop_uptime, gtop_load, gtop_net;
 
 #define GTOP_OFF(el) offsetof(gtop_struct, el)
 
@@ -293,24 +300,24 @@ Gtop_data gtop_vars[] =
   { "idletime",         'D', &gtop_uptime,  GTOP_OFF(uptime.idletime)       },
 
   /* glibtop loadavg stats */
-  { "loadavg_running",  'L', &gtop_loadavg, GTOP_OFF(loadavg.nr_running)    },
-  { "loadavg_tasks",    'L', &gtop_loadavg, GTOP_OFF(loadavg.nr_tasks)      },
-  { "loadavg_1m",       'D', &gtop_loadavg, GTOP_OFF(loadavg.loadavg[0])    },
-  { "loadavg_5m",       'D', &gtop_loadavg, GTOP_OFF(loadavg.loadavg[1])    },
-  { "loadavg_15m",      'D', &gtop_loadavg, GTOP_OFF(loadavg.loadavg[2])    },
+  { "load_running",     'L', &gtop_load,    GTOP_OFF(loadavg.nr_running)    },
+  { "load_tasks",       'L', &gtop_load,    GTOP_OFF(loadavg.nr_tasks)      },
+  { "load_1m",          'D', &gtop_load,    GTOP_OFF(loadavg.loadavg[0])    },
+  { "load_5m",          'D', &gtop_load,    GTOP_OFF(loadavg.loadavg[1])    },
+  { "load_15m",         'D', &gtop_load,    GTOP_OFF(loadavg.loadavg[2])    },
 
   /* netload stats -- Linux-specific for the time being */
-  { "net_pkts_in",      'L', &gtop_netload, GTOP_OFF(netload.packets_in)    },
-  { "net_pkts_out",     'L', &gtop_netload, GTOP_OFF(netload.packets_out)   },
-  { "net_pkts_total",   'L', &gtop_netload, GTOP_OFF(netload.packets_total) },
+  { "net_pkts_in",      'L', &gtop_net,     GTOP_OFF(netload.packets_in)    },
+  { "net_pkts_out",     'L', &gtop_net,     GTOP_OFF(netload.packets_out)   },
+  { "net_pkts_total",   'L', &gtop_net,     GTOP_OFF(netload.packets_total) },
 
-  { "net_bytes_in",     'L', &gtop_netload, GTOP_OFF(netload.bytes_in)      },
-  { "net_bytes_out",    'L', &gtop_netload, GTOP_OFF(netload.bytes_out)     },
-  { "net_bytes_total",  'L', &gtop_netload, GTOP_OFF(netload.bytes_total)   },
+  { "net_bytes_in",     'L', &gtop_net,     GTOP_OFF(netload.bytes_in)      },
+  { "net_bytes_out",    'L', &gtop_net,     GTOP_OFF(netload.bytes_out)     },
+  { "net_bytes_total",  'L', &gtop_net,     GTOP_OFF(netload.bytes_total)   },
 
-  { "net_errs_in",      'L', &gtop_netload, GTOP_OFF(netload.errors_in)     },
-  { "net_errs_out",     'L', &gtop_netload, GTOP_OFF(netload.errors_out)    },
-  { "net_errs_total",   'L', &gtop_netload, GTOP_OFF(netload.errors_total)  },
+  { "net_errs_in",      'L', &gtop_net,     GTOP_OFF(netload.errors_in)     },
+  { "net_errs_out",     'L', &gtop_net,     GTOP_OFF(netload.errors_out)    },
+  { "net_errs_total",   'L', &gtop_net,     GTOP_OFF(netload.errors_total)  },
 
   /* end of array marker */
   { NULL,	         0,  NULL,          0 }
@@ -347,11 +354,11 @@ gtop_value(
 	    }
 	  else /* if (gtop_vars[i].type == 'L') */
 	    {
-	      unsigned long ul = *((unsigned long *)cp);
+	      u_int64_t ul = *((u_int64_t *)cp);
 	      if (delta)
 		{
 		  cp = ((char *)gtp_last) + gtop_vars[i].off;
-		  ul -= *((unsigned long *)cp);
+		  ul -= *((unsigned long long *)cp);
 		}
 	      *val = ul;
 	    }
@@ -363,20 +370,29 @@ gtop_value(
 
 /*
  * get_all_netload -- a Linux-specific routine to get net load
- * information for all interfaces
+ * information for all interfaces.  This assumes that u_int64_t is
+ * defined and that 64-bit ints are available, but since this is quite
+ * linux-specific anyway, I think that this should be okay. 
  */
 static void
 get_all_netload(glibtop_netload *netload)
 {
   FILE *fd;
   memset(netload, 0, sizeof(*netload));
+#if linux
   if ((fd = fopen("/proc/net/dev", "r")) != NULL)
     {
-      unsigned long bytes_in, pkts_in, errs_in, bytes_out, pkts_out, errs_out;
+      u_int64_t bytes_in, pkts_in, errs_in, bytes_out, pkts_out, errs_out;
       fscanf(fd, "%*[^\n]\n%*[^\n]\n");
+#if G_MAXLONG == 9223372036854775807
       while (fscanf(fd,
 	"%*[^:]:%ld%ld%ld%*d%*d%*d%*d%*d%ld%ld%ld%*d%*d%*d%*d%*d",
 	&bytes_in, &pkts_in, &errs_in, &bytes_out, &pkts_out, &errs_out) == 6)
+#else
+      while (fscanf(fd,
+	"%*[^:]:%lld%lld%lld%*d%*d%*d%*d%*d%lld%lld%lld%*d%*d%*d%*d%*d",
+	&bytes_in, &pkts_in, &errs_in, &bytes_out, &pkts_out, &errs_out) == 6)
+#endif
 	{
 	  netload->packets_in    += pkts_in;
 	  netload->packets_out   += pkts_out;
@@ -390,6 +406,7 @@ get_all_netload(glibtop_netload *netload)
 	}
       fclose(fd);
     }
+#endif
 }
 #endif
 
@@ -543,7 +560,6 @@ static double eval(
   e.gtp_now = gtp_now;
   e.gtp_last = gtp_last;
 #endif
-
   if (setjmp(e.err_jmp))
     return e.val;
 
@@ -558,6 +574,7 @@ static double eval(
 /*
  * Param -- struct describing a single parameter.
  */
+typedef enum { scale_linear, scale_log } scale_t;
 typedef struct
 {
   char active;		/* set if this parameter should be evaluated */
@@ -579,6 +596,7 @@ typedef struct
   int vars;		/* how many vars to read from the line in the file */
   double *last, *now;	/* the last and current vars, as enumerated above */
   float *val;		/* value history */
+  scale_t scale;	/* function to scale value by prior to display */
 }
 Param;
 
@@ -609,12 +627,13 @@ static Param_glob chart_glob, slider_glob;
 static void no_display(void);
 static void numeric_with_ident(void);
 static void numeric_with_graph(void);
-static void gtk_graph(void);
+static void gnome_graph(void);
+static void applet(void);
 
 /*
  * The display variable points to the display proccessing routine.
  */
-static void (*display)(void) = gtk_graph;
+static void (*display)(void) = gnome_graph;
 
 /*
  * defns_error -- reports error and exits during config file parsing.
@@ -622,15 +641,21 @@ static void (*display)(void) = gtk_graph;
 static void
 defns_error(char *fn, int ln, char *fmt, ...)
 {
+  int len = 0;
+  char msg[1000];
   va_list args;
   fflush(stdout);
   va_start(args, fmt);
-  fprintf(stderr, "%s: ", prog_name);
+  len = sprintf(msg, "%s: ", prog_name);
   if (fn)
-    fprintf(stderr, _("%s, line %d: "), fn, ln);
-  vfprintf(stderr, fmt, args);
-  fprintf(stderr, "\n");
+    len += sprintf(msg + len, _("%s, line %d: "), fn, ln);
+  vsprintf(msg + len, fmt, args);
+  fprintf(stderr, "%s\n", msg);
   va_end(args);
+
+  if (graphic_display)
+    gnome_dialog_run(GNOME_DIALOG(gnome_error_dialog(msg)));
+
   exit(EXIT_FAILURE);
 }
 
@@ -781,20 +806,6 @@ read_param_defns(Param_glob *pgp)
 		minor_tick = atoi(val);
 	      else if (streq(key, "major_ticks"))
 		major_tick = atoi(val);
-	      else if (streq(key, "type"))
-		{
-		  if (streq("none", val))
-		    display = no_display;
-		  else if (streq("text", val))
-		    display = numeric_with_ident;
-		  else if (streq("graph", val))
-		    display = numeric_with_graph;
-		  else if (streq("gtk", val))
-		    display = gtk_graph;
-		  else
-		    defns_error(
-		      fn, lineno, _("invalid display type: %s"), val);
-		}
 	      /* An "identifier" or "begin" keyword introduces a new
                  parameter.  We bump the params count, and allocate
                  and initialize a new Param struct with default
@@ -819,10 +830,11 @@ read_param_defns(Param_glob *pgp)
 		  p[params-1]->led_gc = NULL;
 		  p[params-1]->hi = p[params-1]->top = 1.0;
 		  p[params-1]->lo = p[params-1]->bot = 0.0;
+		  p[params-1]->scale = scale_linear;
 		}
 	      else if (streq(key, "end"))
 		{
-		  if (!streq(p[params-1]->ident, val))
+		  if (val && !streq(p[params-1]->ident, val))
 		    defns_error(fn, lineno, "found %s when expecting %s",
 		      val, p[params-1]->ident);
 		}
@@ -833,8 +845,9 @@ read_param_defns(Param_glob *pgp)
 	      else if (streq(key, "color"))
 		{
 		  p[params-1]->color_name = strdup(val);
-		  if (!gdk_color_parse(val, &p[params-1]->gdk_color))
-		    defns_error(fn, lineno, _("unrecognized color: %s"), val);
+		  if (graphic_display)
+		    if (!gdk_color_parse(val, &p[params-1]->gdk_color))
+		      defns_error(fn,lineno, _("unrecognized color: %s"), val);
 		}
 	      else if (streq(key, "lights"))
 		{
@@ -847,16 +860,27 @@ read_param_defns(Param_glob *pgp)
 		      trimtb(cname);
 		      pp->led_color = realloc(
 			pp->led_color, (n+1) * sizeof(*pp->led_color));
-		      if (!gdk_color_parse(cname, &pp->led_color[n]))
-			{
-			  defns_error(
-			    fn, lineno, _("unrecognized color: %s"), cname);
-			}
+		      if (graphic_display)
+			if (!gdk_color_parse(cname, &pp->led_color[n]))
+			  {
+			    defns_error(
+			      fn, lineno, _("unrecognized color: %s"), cname);
+			  }
 		      free(cname);
 		    }
 		  pp->num_leds = n;
 		  pp->is_led = 1;
 		}
+	      else if (streq(key, "scale"))
+		{
+		  if (val && streq(val, "linear"))
+		    p[params-1]->scale = scale_linear;
+		  else if (val && streq(val, "log"))
+		    p[params-1]->scale = scale_log;
+		  else
+		    defns_error(fn, lineno,
+		      _("scale must be either \"log\" or \"linear\""));
+		}
 	      else if (streq(key, "active"))
 		p[params-1]->active = yes_no(val);
 	      else if (streq(key, "filename"))
@@ -867,10 +891,20 @@ read_param_defns(Param_glob *pgp)
 		p[params-1]->vars = atoi(val);
 	      else if (streq(key, "equation"))
 		{
-		  char src[FILENAME_MAX+20];
+		  char *cp = val, src[FILENAME_MAX+20];
 		  sprintf(src, "%s, line %d", fn, lineno);
+		  if (!val)
+		    defns_error(fn, lineno, _("no equation found"));
 		  p[params-1]->eqn_src = strdup(src);
 		  p[params-1]->eqn = strdup(val);
+		  /* If the number of fields to extract from the data
+                     file has not been specified, use the largest int
+                     following a "$" or "~". */
+		  if (p[params-1]->vars == 0)
+		    while (*(cp += strcspn(cp, "$~")))
+		      if (isdigit(*++cp))
+			if (p[params-1]->vars < atoi(cp))
+			  p[params-1]->vars = atoi(cp);
 		}
 	      else if (streq(key, "maximum"))
 		p[params-1]->hi = p[params-1]->top = atof(val);
@@ -956,6 +990,30 @@ cap(double x)
   return ranges[j] * d;
 }
 
+#ifdef HAVE_LIBGTOP
+/*
+ * glibtop_get_swap_occasionally -- like glibtop_get_swap, except that
+ * glibtop_get_swap only gets called at most once every 15 seconds.
+ * This ugly hack was put in because on Linux, reading /proc/meminfo
+ * to get swap informetion takes ~ 10ms, and this was causing
+ * gstripchart to chew up significant CPU time.
+ */
+static void
+glibtop_get_swap_occasionally(glibtop_swap *swap)
+{
+  time_t time_now = time(NULL);
+  static time_t time_last;
+  static glibtop_swap swap_last;
+
+  if ((time_now - time_last) > 15)
+    {
+      glibtop_get_swap(&swap_last);
+      time_last = time_now;
+    }
+  *swap = swap_last;
+}
+#endif
+
 static void
 update_values(Param_glob *pgp, Param_glob *slave_pgp)
 {
@@ -973,12 +1031,12 @@ update_values(Param_glob *pgp, Param_glob *slave_pgp)
   if (gtop_mem)
     glibtop_get_mem(&pgp->gtop_now.mem);
   if (gtop_swap)
-    glibtop_get_swap(&pgp->gtop_now.swap);
+    glibtop_get_swap_occasionally(&pgp->gtop_now.swap);
   if (gtop_uptime)
     glibtop_get_uptime(&pgp->gtop_now.uptime);
-  if (gtop_loadavg)
+  if (gtop_load)
     glibtop_get_loadavg(&pgp->gtop_now.loadavg);
-  if (gtop_netload)
+  if (gtop_net)
     get_all_netload(&pgp->gtop_now.netload);
 #endif
 
@@ -1149,10 +1207,6 @@ readjust_top_for_width(int width)
 	  chart_glob.parray[p]->top = top;
 	  if (include_slider)
 	    slider_glob.parray[p]->top = top;
-#if 0
-	  printf("top[%d] %s\t= %g\t=> %g\n",
-	    p, chart_glob.parray[p]->ident, t, chart_glob.parray[p]->top);
-#endif
 	  readjust = 1;
 	}
     }
@@ -1166,10 +1220,12 @@ static GdkPixmap *pixmap;
 static GdkColormap *colormap;
 
 static gint
-config_handler(GtkWidget *widget, GdkEventConfigure *e, gpointer whence)
+config_handler(GtkWidget *widget, GdkEventConfigure *not_used, gpointer unused)
 {
   int p, c, w = widget->allocation.width, h = widget->allocation.height;
-
+#ifdef DEBUG
+  printf("config: %p, %dx%d\n", widget, w, h);
+#endif
   /* On the initial configuration event, get the window colormap and
      allocate a color in it for each parameter color. */
   if (colormap == NULL)
@@ -1266,9 +1322,14 @@ overlay_status_box(GtkWidget *widget)
 /*
  * val2y -- scales a parameter value into a y coordinate value.
  */
+#ifndef HAVE_LOGF
+#define logf(x) ((float)log(x))
+#endif
 static int
-val2y(float val, float top, int height)
+val2y(float val, float top, int height, scale_t scale)
 {
+  if (scale == scale_log)
+    return height - (height * logf(1+val) / logf(1+top));
   return height - (height * val / top);
 }
 
@@ -1284,16 +1345,17 @@ chart_expose_handler(GtkWidget *widget, GdkEventExpose *event)
     if (chart_glob.parray[p]->active && !chart_glob.parray[p]->is_led)
       {
 	float top = chart_glob.parray[p]->top;
+	scale_t scale = chart_glob.parray[p]->scale;
 	int n = w < chart_glob.num_val ? w : chart_glob.num_val;
 	int i, j = chart_glob.new_val;
 	int x0, x1 = w - 1;
-	int y0, y1 = val2y(chart_glob.parray[p]->val[j], top, h);
+	int y0, y1 = val2y(chart_glob.parray[p]->val[j], top, h, scale);
 	for (i=0; i<n; i++)
 	  {
 	    if (--j < 0) j = chart_glob.max_val - 1;
 	    x0 = x1; y0 = y1;
 	    x1 = x0 - 1;
-	    y1 = val2y(chart_glob.parray[p]->val[j], top, h);
+	    y1 = val2y(chart_glob.parray[p]->val[j], top, h, scale);
 	    gdk_draw_line(pixmap, chart_glob.parray[p]->gdk_gc, x0,y0, x1,y1);
 	  }
       }
@@ -1311,6 +1373,8 @@ chart_expose_handler(GtkWidget *widget, GdkEventExpose *event)
   return FALSE;
 }
 
+static void text_popup_refresh(void);
+
 static gint
 chart_timer_handler(GtkWidget *widget)
 {
@@ -1335,18 +1399,20 @@ chart_timer_handler(GtkWidget *widget)
       /* Shift the pixmap one pixel left, and clear the RHS  */
       gdk_window_copy_area(
 	pixmap, widget->style->fg_gc[GTK_WIDGET_STATE(widget)], 0, 0,
-	pixmap, 1, 0, w-1, h);
+	pixmap, 1,0, w-1,h);
       gdk_draw_rectangle(
 	pixmap, widget->style->bg_gc[GTK_WIDGET_STATE(widget)],
-	TRUE, w-1, 0, 1, h);
+	TRUE, w-1,0, 1,h);
       for (p = chart_glob.params; p--; )
 	if (chart_glob.parray[p]->active && !chart_glob.parray[p]->is_led)
 	  {
 	    float top = chart_glob.parray[p]->top;
+	    scale_t scale = chart_glob.parray[p]->scale;
 	    int i = chart_glob.new_val;
 	    int m = chart_glob.max_val;
-	    int y1 = val2y(chart_glob.parray[p]->val[i], top, h);
-	    int y0 = val2y(chart_glob.parray[p]->val[(i+m-1) % m], top, h);
+	    int y1 = val2y(chart_glob.parray[p]->val[i], top, h, scale);
+	    int y0 = val2y(chart_glob.parray[p]->val[(i+m-1) % m],
+	      top, h, scale);
 	    gdk_draw_line(pixmap, chart_glob.parray[p]->gdk_gc,
 	      w-2,y0, w-1,y1);
 	  }
@@ -1356,6 +1422,8 @@ chart_timer_handler(GtkWidget *widget)
       overlay_tick_marks(widget, minor_tick, major_tick);
       overlay_status_box(widget);
     }
+
+  text_popup_refresh();
   return TRUE;
 }
 
@@ -1371,7 +1439,7 @@ slider_redraw(GtkWidget *widget)
 	GdkPoint tri[3];
 	int y = val2y(
 	  slider_glob.parray[p]->val[slider_glob.new_val],
-	  slider_glob.parray[p]->top, h);
+	  slider_glob.parray[p]->top, h, slider_glob.parray[p]->scale);
 	tri[0].x = 0; tri[0].y = y;
 	tri[1].x = w; tri[1].y = y-w/2;
 	tri[2].x = w; tri[2].y = y+w/2;
@@ -1422,7 +1490,7 @@ about_callback(void)
   const gchar *authors[] = { "John Kodis, kodis@jagunet.com", NULL };
   GtkWidget *about = gnome_about_new(
     _(prog_name), prog_version,
-    _("Copyright 1998 John Kodis"),
+    _("Copyright 1998,99 John Kodis"),
     authors,
     _("The GNOME stripchart program plots various user-specified parameters "
       "as a function of time.  Its main use is to chart system performance "
@@ -1458,7 +1526,7 @@ GnomeUIInfo mainmenu[] =
  * and points it toward the gstripchart help file.
  */
 static void
-help_menu_action(GtkWidget *menu)
+help_menu_action(GtkWidget *unused1, void *unused2)
 {
   static GnomeHelpMenuEntry help_entry = { "gstripchart", "index.html" };
   gnome_help_display(NULL, &help_entry);
@@ -1501,7 +1569,7 @@ prefs_okay(GtkWidget *button, gpointer dialog)
  * chart parameters.
  */
 static gint
-prefs_callback(GtkWidget *chart, gpointer unused)
+prefs_callback(GtkWidget *chart, void *unused)
 {
   int p;
   GtkWidget *dialog, *notebook, *vbox, *clist, *active, *label;
@@ -1558,7 +1626,7 @@ prefs_callback(GtkWidget *chart, gpointer unused)
     }
   gtk_widget_show(notebook);
 
-  dialog = gnome_dialog_new(_("Gnome Stripchart Parameters"),
+  dialog = gnome_dialog_new(_("Stripchart Parameters"),
     GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_APPLY,
     GNOME_STOCK_BUTTON_CANCEL, GNOME_STOCK_BUTTON_HELP, NULL);
   gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(chart));
@@ -1580,69 +1648,102 @@ prefs_callback(GtkWidget *chart, gpointer unused)
  * text_load_clist -- fills a clist with chart ident, current, and top values.
  */
 static void
-text_load_clist(GtkWidget *txt, GtkWidget *box)
+text_load_clist(GtkWidget *text_popup_clist, GtkWidget *text_popup_window)
 {
   int n;
-  gtk_clist_freeze(GTK_CLIST(txt));
-  gtk_clist_clear(GTK_CLIST(txt));
+  gtk_clist_freeze(GTK_CLIST(text_popup_clist));
+  gtk_clist_clear(GTK_CLIST(text_popup_clist));
   for (n = 0; n < chart_glob.params; n++)
     {
       Param *p = chart_glob.parray[n];
       char val_str[100], top_str[100];
-      char *row_strs[3];
+      char *row_strs[4];
 
       row_strs[0] = _(p->ident);
       row_strs[1] = val_str;
       row_strs[2] = top_str;
+      row_strs[3] = p->is_led ? "*" : p->scale ? "log" : "-";
       hi_lo_fmt(p->top, top_str, p->val[chart_glob.new_val], val_str);
-      gtk_clist_append(GTK_CLIST(txt), row_strs);
-      gtk_clist_set_foreground(GTK_CLIST(txt), n, &p->gdk_color);
-      gtk_clist_set_background(GTK_CLIST(txt), n, &box->style->bg[0]);
+      gtk_clist_append(GTK_CLIST(text_popup_clist), row_strs);
+      gtk_clist_set_foreground(GTK_CLIST(text_popup_clist),
+	n, &p->gdk_color);
+      gtk_clist_set_background(GTK_CLIST(text_popup_clist),
+	n, &text_popup_window->style->bg[0]);
     }
-  gtk_clist_columns_autosize(GTK_CLIST(txt));
-  gtk_clist_thaw(GTK_CLIST(txt));
+  gtk_clist_columns_autosize(GTK_CLIST(text_popup_clist));
+  gtk_clist_thaw(GTK_CLIST(text_popup_clist));
 }
 
 /*
- * text_update -- updates the text box values on response to a box click.
+ * text_popup widgets: these are file-scoped so that the text popup
+ * window can be refreshed from the chart timer handler.
+ */
+static GtkWidget *text_popup_window, *text_popup_clist;
+
+/*
+ * text_popup_refresh -- if the text popup window is up, refresh the
+ * values.  Called from chart_timer_handler().
  */
 static void
-text_update(GtkWidget *box, GdkEvent *event, GtkWidget *txt)
+text_popup_refresh(void)
 {
-  text_load_clist(txt, box);
+  if (text_popup_window)
+    text_load_clist(text_popup_clist, text_popup_window);
+}
+
+/*
+ * text_popup_close -- destroy event handler.  Destroys the window.
+ */
+static void
+text_popup_close(void)
+{
+  if (text_popup_window)
+    {
+      gtk_widget_destroy(GTK_WIDGET(text_popup_window));
+      text_popup_window = NULL;
+    }
 }
 
 /*
  * text_popup -- pops up a top level textual display of current
- * parameter values in response to a mouse click.
+ * parameter values in response to a mouse click.  Closes this window
+ * if it's already displayed.
  */
 static void
 text_popup(GtkWidget *widget, GdkEvent *event)
 {
-  static GtkWidget *box, *txt;
   GdkEventButton *button = (GdkEventButton*)event;
 
-  if (box)
-    {
-      gtk_widget_destroy(GTK_WIDGET(box));
-      box = NULL;
-    }
+  if (text_popup_window)
+    text_popup_close();
   else
     {
-      char *titles[3];
-      titles[0] = _("Param"); titles[1] = _("Current"); titles[2] = _("Top");
-      txt = gtk_clist_new_with_titles(NELS(titles), titles);
-      gtk_widget_show(txt);
+      char *titles[4];
+      titles[0] = _("Param");
+      titles[1] = _("Current");
+      titles[2] = _("Top");
+      titles[3] = _("Scale");
+      text_popup_clist = gtk_clist_new_with_titles(NELS(titles), titles);
+      gtk_widget_show(text_popup_clist);
 
-      box = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-      gtk_widget_set_style(GTK_WIDGET(box), widget->style);
-      gtk_container_add(GTK_CONTAINER(box), txt);
-      gtk_window_set_transient_for(GTK_WINDOW(box), GTK_WINDOW(widget));
-      gtk_widget_set_uposition(GTK_WIDGET(box), button->x_root,button->y_root);
-      gtk_signal_connect(GTK_OBJECT(box),
-	"button_press_event", GTK_SIGNAL_FUNC(text_update), txt);
-      text_load_clist(txt, widget);
-      gtk_widget_show(box);
+      text_popup_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+      gtk_widget_set_style(GTK_WIDGET(text_popup_window), widget->style);
+      gtk_container_add(GTK_CONTAINER(text_popup_window), text_popup_clist);
+      gtk_window_set_transient_for(GTK_WINDOW(text_popup_window),
+	GTK_WINDOW(widget));
+      gtk_widget_set_uposition(GTK_WIDGET(text_popup_window),
+	button->x_root, button->y_root);
+      gtk_widget_show(text_popup_window);
+
+      /* Load the initial batch of parameter values after showing the
+         toplevel window.  This odd sequence is required to get the
+         auto-sizing right. */
+      text_load_clist(text_popup_clist, widget);
+
+      /* Handle any button press events by updating the parameter
+         values in the clist popup. */
+      gtk_signal_connect(GTK_OBJECT(text_popup_window),
+	"destroy", GTK_SIGNAL_FUNC(text_popup_close), NULL);
     }
 }
 
@@ -1730,18 +1831,17 @@ save_handler(GnomeClient *client,
 }
 
 /*
- * gtk_graph -- the display processor for the main, Gtk-based
+ * gnome_graph -- the display processor for the main, Gtk-based
  * graphical display. 
  */
 static void
-gtk_graph(void)
+gnome_graph(void)
 {
   GnomeClient *client;
   GtkWidget *frame, *h_box, *drawing;
-  const int slide_w=10;		/* Set the slider width. */
 
-  /* Create a drawing area.  Add it to the window, show it, and
-     set its expose event handler. */
+  /* Create a drawing area for the chart plot; pack it into an h-box,
+     and connect the configure and expose signal handlers. */
   drawing = gtk_drawing_area_new();
   gtk_drawing_area_size(GTK_DRAWING_AREA(drawing), geometry_w, geometry_h);
   gtk_widget_set_events(drawing, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK);
@@ -1751,9 +1851,13 @@ gtk_graph(void)
   gtk_box_pack_start(GTK_BOX(h_box), drawing, TRUE, TRUE, 0);
 
   gtk_signal_connect(GTK_OBJECT(drawing),
-    "configure_event", (GtkSignalFunc)config_handler, "draw");
+    "configure_event", (GtkSignalFunc)config_handler, NULL);
   gtk_signal_connect(GTK_OBJECT(drawing),
     "expose_event", (GtkSignalFunc)chart_expose_handler, NULL);
+
+  /* If a slider window is to be included, create a drawing area for
+     that, pack it and separator into the h-box, connect the expose
+     handler and start an update timer. */
   if (include_slider)
     {
       GtkWidget *sep = gtk_vseparator_new();
@@ -1762,7 +1866,8 @@ gtk_graph(void)
       gtk_box_pack_start(GTK_BOX(h_box), sep, FALSE, TRUE, 0);
       gtk_widget_show(sep);
 
-      gtk_drawing_area_size(GTK_DRAWING_AREA(slider), slide_w, geometry_h);
+      gtk_drawing_area_size(GTK_DRAWING_AREA(slider),
+	slider_width, geometry_h);
       gtk_box_pack_start(GTK_BOX(h_box), slider, FALSE, FALSE, 0);
       gtk_widget_show(slider);
 
@@ -1775,10 +1880,10 @@ gtk_graph(void)
     }
   gtk_widget_show(h_box);
 
-  /* Create a top-level window. Set the title, minimum size (_usize),
+  /* Create a top-level application window. Set minimum size (_usize),
      initial size (_default_size), and establish delete and destroy
      event handlers. */
-  frame = gnome_app_new(_("gstripchart"), _("Gnome stripchart viewer"));
+  frame = gnome_app_new(_("gstripchart"), _("Stripchart Viewer"));
   gtk_widget_set_usize(frame, 1, 1); /* min_w, min_h */
   gtk_window_set_default_size(GTK_WINDOW(frame), geometry_w, geometry_h);
   gtk_signal_connect(GTK_OBJECT(frame),
@@ -1794,7 +1899,7 @@ gtk_graph(void)
     gnome_app_create_menus(GNOME_APP(frame), mainmenu);
   gnome_app_set_contents(GNOME_APP(frame), h_box);
 
-  /* Create timer events for the drawing and slider widgets. */
+  /* Connect a timer event to update the chart drawing area. */
   gtk_timeout_add((int)(1000 * chart_interval),
     (GtkFunction)chart_timer_handler, drawing);
 
@@ -1805,7 +1910,7 @@ gtk_graph(void)
   if (geometry_flags & (XNegative | YNegative))
     {
       if (XNegative)
-	geometry_x = root_width  + geometry_x - geometry_w - slide_w;
+	geometry_x = root_width  + geometry_x - geometry_w - slider_width;
       if (YNegative)
 	geometry_y = root_height + geometry_y - geometry_h;
     }
@@ -1827,6 +1932,138 @@ gtk_graph(void)
   gtk_main();
 }
 
+#ifdef HAVE_GNOME_APPLET
+static void
+applet_orient_handler(AppletWidget *applet, int i, GtkDrawingArea *drawings[2])
+{
+  int o, h, w;
+  o = applet_widget_get_panel_orient(applet);
+#ifdef HAVE_PANEL_PIXEL_SIZE
+  h = applet_widget_get_panel_pixel_size(applet);
+#else
+  h = 48;
+#endif
+  if (o == ORIENT_UP || o == ORIENT_DOWN)
+    w = h * 3;
+  else
+    w = include_slider ? h - (slider_width + 2) : h;
+  gtk_drawing_area_size(drawings[0], w, h);
+  if (drawings[1])
+    gtk_drawing_area_size(drawings[1], slider_width, h);
+#ifdef DEBUG
+  printf("orient: %dx%d, %c(%d), %d\n", w, h, "UDLR"[o], o, i);
+#endif
+}
+
+#ifdef HAVE_PANEL_PIXEL_SIZE
+static void
+applet_resize_handler(AppletWidget *applet, int i, GtkDrawingArea *drawings[2])
+{
+  int o, h, w;
+  o = applet_widget_get_panel_orient(applet);
+  h = applet_widget_get_panel_pixel_size(applet);
+  if (o == ORIENT_UP || o == ORIENT_DOWN)
+    w = h * 3;
+  else
+    w = include_slider ? h - (slider_width + 2) : h;
+  gtk_drawing_area_size(drawings[0], w, h);
+  if (drawings[1])
+    gtk_drawing_area_size(drawings[1], slider_width, h);
+#ifdef DEBUG
+  printf("resize: %dx%d, %c(%d), %d\n", w, h, "UDLR"[o], o, i);
+#endif
+}
+#endif
+
+/*
+ * applet -- a display routine like gnome_graph, but in an applet
+ * rather than a gnome frame. 
+ */
+static void
+applet(void)
+{
+  GtkWidget *frame, *h_box, *drawing, *drawings[2];
+
+  /* Create a too-small drawing area to plot the chart data in, and
+     connect the configure and expose signal handlers.  The drawing
+     area gets reconfigured to its proper size when the change-size or
+     change-orient handlers get run.  I'd rather have just created an
+     empty applet frame here, and populate it in the change-size
+     handler; unfortunately, the change-size signal doesn't get sent
+     if the applet is left empty.  Hence, this hack.  */
+  drawings[0] = drawing = gtk_drawing_area_new();
+  gtk_drawing_area_size(GTK_DRAWING_AREA(drawing), 15, 15);
+  gtk_widget_set_events(drawing, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK);
+  gtk_signal_connect(GTK_OBJECT(drawing),
+    "configure_event", (GtkSignalFunc)config_handler, NULL);
+  gtk_signal_connect(GTK_OBJECT(drawing),
+    "expose_event", (GtkSignalFunc)chart_expose_handler, NULL);
+  gtk_widget_show(drawing);
+
+  /* Pack the chart into an h-box.  If the sliders are to be
+     displayed, create a drawing area for the slider box and pack that
+     in too.  */
+  h_box = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(h_box), drawing, TRUE, TRUE, 0);
+  if (include_slider)
+    {
+      GtkWidget *sep = gtk_vseparator_new();
+      GtkWidget *slider = gtk_drawing_area_new();
+      drawings[1] = slider;
+
+      gtk_box_pack_start(GTK_BOX(h_box), sep, FALSE, TRUE, 0);
+      gtk_widget_show(sep);
+
+      gtk_drawing_area_size(GTK_DRAWING_AREA(slider), slider_width, 15);
+      gtk_box_pack_start(GTK_BOX(h_box), slider, FALSE, FALSE, 0);
+      gtk_widget_show(slider);
+
+      gtk_widget_set_events(slider, GDK_EXPOSURE_MASK);
+      gtk_signal_connect(GTK_OBJECT(slider),
+        "expose_event", (GtkSignalFunc)slider_expose_handler, NULL);
+
+      gtk_timeout_add((int)(1000 * slider_interval),
+        (GtkFunction)slider_timer_handler, slider);
+    }
+  gtk_timeout_add((int)(1000 * chart_interval),
+    (GtkFunction)chart_timer_handler, drawing);
+  gtk_widget_show(h_box);
+
+  /* Create an applet frame and connect the various signal handlers
+     and menu items.  */
+  frame = applet_widget_new(applet_name);
+  gtk_signal_connect(GTK_OBJECT(frame),
+    "destroy", GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
+  gtk_signal_connect(GTK_OBJECT(frame),
+    "delete_event", GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
+#ifdef HAVE_PANEL_PIXEL_SIZE
+  gtk_signal_connect(GTK_OBJECT(frame),
+    "change_pixel_size", (GtkSignalFunc)applet_resize_handler, drawings);
+#endif
+  gtk_signal_connect(GTK_OBJECT(frame),
+    "change_orient", (GtkSignalFunc)applet_orient_handler, drawings);
+  gtk_signal_connect(GTK_OBJECT(frame),
+    "button_press_event", GTK_SIGNAL_FUNC(click_handler), NULL);
+
+  applet_widget_register_stock_callback(APPLET_WIDGET(frame),
+    "help", GNOME_STOCK_MENU_BLANK, _("Help"),
+    (AppletCallbackFunc)help_menu_action, NULL);
+  applet_widget_register_stock_callback(APPLET_WIDGET(frame),
+    "about", GNOME_STOCK_MENU_ABOUT, _("About"),
+    (AppletCallbackFunc)about_callback, NULL);
+  applet_widget_register_stock_callback(APPLET_WIDGET(frame),
+    "params", GNOME_STOCK_MENU_BLANK, _("Params..."),
+    (AppletCallbackFunc)prefs_callback, NULL);
+
+  /* Add the h-box to the applet, and off we go. */
+  applet_widget_add(APPLET_WIDGET(frame), h_box);
+  gtk_widget_show(frame);
+  applet_widget_gtk_main();
+}
+#else
+static void applet(void) { gnome_graph(); }
+#endif
+
 static int
 proc_arg(int opt, const char *arg)
 {
@@ -1840,15 +2077,18 @@ proc_arg(int opt, const char *arg)
     case 'M': include_menubar = 1; break;
     case 'S': include_slider = 0; break;
     case 'g':
-      geometry_flags = XParseGeometry(arg,
-	&geometry_x, &geometry_y, &geometry_w, &geometry_h);
+      geometry_flags = XParseGeometry(
+	arg, &geometry_x, &geometry_y, &geometry_w, &geometry_h);
       break;
     case 't':
       if (streq("gtk", arg))
-	display = gtk_graph;
+	display = gnome_graph;
+      else if (streq("applet", arg))
+	display = applet;
       else
 	{
 	  gnome_client_disable_master_connection();
+	  graphic_display = 0;
 	  if (streq("none", arg))
 	    display = no_display;
 	  else if (streq("text", arg))
@@ -1872,10 +2112,8 @@ popt_arg_extractor(
 {
   if (proc_arg(opt->val, arg))
     {
-      /* FIX THIS: the program name includes trailing junk, and
-       * although the long options aren't shown, all of the Gnome
-       * internal options are. */
       poptPrintUsage(state, stderr, 0);
+      exit(EXIT_FAILURE);
     }
 }
 
@@ -1900,11 +2138,14 @@ poptOption arglist[] =
   { "omit-slider",     'S', POPT_ARG_NONE, NULL, 'S',
     N_("Omits slider window"), NULL },
   { "display-type",    't', POPT_ARG_STRING, NULL, 't',
-    N_("TYPE is one of gtk, text, graph, or none"), N_("TYPE") },
+#ifdef HAVE_GNOME_APPLET
+    N_("TYPE is one of gtk, applet, text, or none"), N_("TYPE") },
+#else
+    N_("TYPE is one of gtk, text, or none"), N_("TYPE") },
+#endif
   { NULL,             '\0', 0, NULL, 0 }
 };
 
-
 /*
  * main -- for the stripchart display program.
  */
@@ -1914,42 +2155,75 @@ main(int argc, char **argv)
   int c;
   poptContext popt_context;
 
-  /* Initialize the i18n stuff.  If the gtop library is linked in,
-     initialize that as well.  Let gnome_init initialize the Gnome and
-     Gtk stuff. */
+  /* Initialize the i18n code, and the gtop library if it's linked in.  */
   bindtextdomain(PACKAGE, GNOMELOCALEDIR);
   textdomain(PACKAGE);
 #ifdef HAVE_LIBGTOP
   glibtop_init_r(&glibtop_global_server, 0, 0);
 #endif
 
-  /* Arrange for gnome_init to parse the command line options.  The
-     only option that matters here is the configuration filename.  We
-     then process whatever configuration file is set, either by
-     default or as specified on the command line. */
-  gnome_init_with_popt_table(
-    prog_name, prog_version, argc, argv, arglist, 0, &popt_context);
+#ifdef HAVE_GNOME_APPLET
+  /* UGLY_HACK: default to --type=applet if invoked with a program
+     name containing "_applet". */
+  if (strstr(argv[0], applet_name))
+    display = applet;
+  /* EVEN_UGLIER_HACK: call the goad_register_arguments() unpublished
+     internal function to allow the popt argument parser to deal with
+     the --activate-goad-server option. */
+  {
+    extern void goad_register_arguments(void);
+    goad_register_arguments();
+  }
+#endif
 
-  root_width = 1;
-  if (display == gtk_graph)
-    gdk_window_get_geometry(NULL, NULL, NULL, &root_width, &root_height, NULL);
+  /* Make an initial scan of the command line arguments to get the
+     configuration file name and the type of display requested. */
+  popt_context = poptGetContext(prog_name, argc, argv, arglist, 0);
+  while ((c = poptGetNextOpt(popt_context)) >= 0)
+    proc_arg(c, poptGetOptArg(popt_context));
+
+  /* Now we know the display type, and do applet initialization for
+     applets and standard gnome initialization for anything else.  We
+     have to do one or the other so that the color name translation
+     won't dump core. */
+#ifdef HAVE_GNOME_APPLET
+  if (display == applet)
+    applet_widget_init(applet_name, VERSION, argc, argv, arglist, 0, NULL);
+#endif
+  if (display == gnome_graph)
+    gnome_init_with_popt_table(
+      prog_name, VERSION, argc, argv, arglist, 0, NULL);
+
+  /* In either of the graphical modes, we get the width of the root
+     window, and use this as the number of samples to be kept in the
+     value history. */
+  if (graphic_display)
+    gdk_window_get_geometry(
+      NULL, NULL, NULL, &root_width, &root_height, NULL);
 
   chart_glob.max_val = root_width;
   chart_glob.new_val = chart_glob.num_val = 0;
   chart_glob.params = read_param_defns(&chart_glob);
   chart_glob.lpf_const = exp(-chart_filter / chart_interval);
+
+  /* This is part of a none-too-satisfactory initialization sequence.
+     During the update_chart==0 pass, the parameter equations are
+     evaluated solely to detect syntax errors and to determine which
+     libgtop routines should be called at the beginning of each
+     update_values pass.  Any equation evaluation errors detected
+     during this pass are fatal. */
+  update_chart = 0;
   update_values(&chart_glob, NULL);
 
   /* Next, we re-parse the command line options so that they'll
      override any values set in the configuration file. */
   popt_context = poptGetContext(prog_name, argc, argv, arglist, 0);
-  while ((c = poptGetNextOpt(popt_context)) != -1)
-    if (c > 0)
-      proc_arg(c, poptGetOptArg(popt_context));
+  while ((c = poptGetNextOpt(popt_context)) >= 0)
+    proc_arg(c, poptGetOptArg(popt_context));
 
   /* Clone chart_param into a new slider_param array, then override
      the next, last, and val arrays and associated sizing values. */
-  if (display == gtk_graph && include_slider)
+  if (include_slider && graphic_display)
     {
       int p;
       slider_glob = chart_glob;
@@ -1975,13 +2249,6 @@ main(int argc, char **argv)
       update_values(&slider_glob, NULL);
     }
 
-  /* This is part of a none-too-satisfactory initialization sequence.
-     During the prior update_chart==0 pass, the parameter equations
-     are evaluated solely to detect syntax errors and to determine
-     which libgtop routines should be called at the beginning of each
-     update_values pass.  Any equation evaluation errors detected
-     during this pass are fatal. */
-
   /* During the update_chart==1 pass, a set of parameter variables is
      gathered, but the resulting parameter values are not evaluated.
      This pass serves only to populate the various *_now values so
@@ -1996,12 +2263,38 @@ main(int argc, char **argv)
   usleep(1000000);
   update_values(&chart_glob, NULL);
 
-  /* During the update_chart==3 pass, each prev value of each
-     parameter is considered valid, and is used in the low-pass
-     filtering of this and subsequent parameter values. */
-  /* update_chart = 3, set by the display() prior to update_values(). */
+  /* During the update_chart==3 pass (set by the display() prior to
+     update_values()), each prev value of each parameter is considered
+     valid, and is used in the low-pass filtering of this and
+     subsequent parameter values. */
   usleep(1000000);
   display();
 
   return EXIT_SUCCESS;
 }
+
+/* This is a placeholder so that strings inherited from the default
+   config file can be seen by xgettext and marked as translatable; and
+   they can be translated for the gstripchart configuration dialog
+   box.
+
+   Of course it is not possible to translate all possible words; but
+   if the user changes the default config file it is probable that he
+   will choose words in his own language (or at least that please
+   him); keeping this ones here allow for the people that don't change
+   the default config file to still have an interface in their
+   language; something people appreciate a lot; more if you think that
+   the newbies are the most likely to a) never touch to the default
+   files, b) appreciate an interface in their language (most of them
+   don't even know English).
+
+   So, please don't remove this. */
+#if 0
+char *placeholder_for_strings_in_config[] =
+{
+  N_("white"), N_("black"), N_("red"), N_("yellow"),
+  N_("blue"), N_("green"), N_("cyan"), N_("magenta"),
+  N_("Busy"), N_("Load"), N_("Swap"), N_("Battery"),
+  N_("Net In"), N_("Net Out"), N_("ppp")
+};
+#endif
